@@ -12,24 +12,24 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import json
 import time
 import uuid
 
 from keystoneclient.common import cms
-from oslo_config import cfg
+from oslo_serialization import jsonutils
 import six
 from six.moves import http_client
 from testtools import matchers
 
 from keystone.common import extension as keystone_extension
+import keystone.conf
 from keystone.tests import unit
 from keystone.tests.unit import default_fixtures
 from keystone.tests.unit import ksfixtures
 from keystone.tests.unit import rest
 from keystone.tests.unit.schema import v2
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 
 
 class CoreApiTests(object):
@@ -385,7 +385,7 @@ class CoreApiTests(object):
                            expected_status=http_client.OK)
 
     def test_error_response(self):
-        """This triggers assertValidErrorResponse by convention."""
+        """Trigger assertValidErrorResponse by convention."""
         self.public_request(path='/v2.0/tenants',
                             expected_status=http_client.UNAUTHORIZED)
 
@@ -447,7 +447,7 @@ class CoreApiTests(object):
         raise NotImplementedError()
 
     def assertNoRoles(self, r):
-        """Helper method to assert No Roles
+        """Helper method to assert No Roles.
 
         This needs to be overridden by child classes
         based on their content type.
@@ -704,7 +704,7 @@ class CoreApiTests(object):
 
 
 class LegacyV2UsernameTests(object):
-    """Tests to show the broken username behavior in V2.
+    """Test to show the broken username behavior in V2.
 
     The V2 API is documented to use `username` instead of `name`.  The
     API forced used to use name and left the username to fall into the
@@ -715,7 +715,7 @@ class LegacyV2UsernameTests(object):
     """
 
     def create_user(self, **user_attrs):
-        """Creates a users and returns the response object.
+        """Create a users and returns the response object.
 
         :param user_attrs: attributes added to the request body (optional)
         """
@@ -990,7 +990,7 @@ class RestfulTestCase(rest.RestfulTestCase):
             self.role_admin['id'])
 
 
-class V2TestCase(RestfulTestCase, CoreApiTests, LegacyV2UsernameTests):
+class V2TestCase(object):
 
     def config_overrides(self):
         super(V2TestCase, self).config_overrides()
@@ -1024,7 +1024,7 @@ class V2TestCase(RestfulTestCase, CoreApiTests, LegacyV2UsernameTests):
 
     def assertValidExtension(self, extension, expected):
         super(V2TestCase, self).assertValidExtension(extension)
-        descriptions = [ext['description'] for ext in six.itervalues(expected)]
+        descriptions = [ext['description'] for ext in expected.values()]
         description = extension.get('description')
         self.assertIsNotNone(description)
         self.assertIn(description, descriptions)
@@ -1257,7 +1257,7 @@ class V2TestCase(RestfulTestCase, CoreApiTests, LegacyV2UsernameTests):
         data_json = cms.cms_verify(signed_text, CONF.signing.certfile,
                                    CONF.signing.ca_certs)
 
-        data = json.loads(data_json)
+        data = jsonutils.loads(data_json)
 
         return (data, token2)
 
@@ -1398,8 +1398,77 @@ class V2TestCase(RestfulTestCase, CoreApiTests, LegacyV2UsernameTests):
             },
             expected_status=http_client.OK)
 
+    def test_enable_or_disable_user(self):
+        token = self.get_scoped_token()
+        user_id = self.user_badguy['id']
 
-class RevokeApiTestCase(V2TestCase):
+        self.assertFalse(self.user_badguy['enabled'])
+
+        def _admin_request(body, status):
+            resp = self.admin_request(
+                method='PUT',
+                path='/v2.0/users/%s/OS-KSADM/enabled' % user_id,
+                token=token,
+                body=body,
+                expected_status=status)
+            return resp
+
+        # Enable the user.
+        body = {'user': {'enabled': True}}
+        resp = _admin_request(body, http_client.OK)
+        self.assertTrue(resp.json['user']['enabled'])
+
+        # Disable the user.
+        body = {'user': {'enabled': False}}
+        resp = _admin_request(body, http_client.OK)
+        self.assertFalse(resp.json['user']['enabled'])
+
+        # Attributes other than `enabled` should still work due to bug 1607751
+        body = {
+            'user': {
+                'description': uuid.uuid4().hex,
+                'name': uuid.uuid4().hex,
+                'enabled': True
+            }
+        }
+        _admin_request(body, http_client.OK)
+
+        #  `enabled` is boolean, type other than boolean is not allowed.
+        body = {'user': {'enabled': uuid.uuid4().hex}}
+        _admin_request(body, http_client.BAD_REQUEST)
+
+
+class V2TestCaseUUID(V2TestCase, RestfulTestCase, CoreApiTests,
+                     LegacyV2UsernameTests):
+
+    def config_overrides(self):
+        super(V2TestCaseUUID, self).config_overrides()
+        self.config_fixture.config(group='token', provider='uuid')
+
+
+class V2TestCaseFernet(V2TestCase, RestfulTestCase, CoreApiTests,
+                       LegacyV2UsernameTests):
+
+    def config_overrides(self):
+        super(V2TestCaseFernet, self).config_overrides()
+        self.config_fixture.config(group='token', provider='fernet')
+        self.useFixture(
+            ksfixtures.KeyRepository(
+                self.config_fixture,
+                'fernet_tokens',
+                CONF.fernet_tokens.max_active_keys
+            )
+        )
+
+    def test_fetch_revocation_list_md5(self):
+        self.skipTest('Revocation lists do not support Fernet')
+
+    def test_fetch_revocation_list_sha256(self):
+        self.skipTest('Revocation lists do not support Fernet')
+
+
+class RevokeApiTestCase(V2TestCase, RestfulTestCase, CoreApiTests,
+                        LegacyV2UsernameTests):
     def config_overrides(self):
         super(RevokeApiTestCase, self).config_overrides()
         self.config_fixture.config(
@@ -1408,21 +1477,19 @@ class RevokeApiTestCase(V2TestCase):
             revoke_by_id=False)
 
     def test_fetch_revocation_list_admin_200(self):
-        self.skipTest('Revoke API disables revocation_list.')
+        self.skip_test_overrides('Revoke API disables revocation_list.')
 
     def test_fetch_revocation_list_md5(self):
-        self.skipTest('Revoke API disables revocation_list.')
+        self.skip_test_overrides('Revoke API disables revocation_list.')
 
     def test_fetch_revocation_list_sha256(self):
-        self.skipTest('Revoke API disables revocation_list.')
+        self.skip_test_overrides('Revoke API disables revocation_list.')
 
 
 class TestFernetTokenProviderV2(RestfulTestCase):
 
     def setUp(self):
         super(TestFernetTokenProviderV2, self).setUp()
-        self.useFixture(ksfixtures.KeyRepository(self.config_fixture))
-
         # Add catalog data
         self.region = unit.new_region_ref()
         self.region_id = self.region['id']
@@ -1458,6 +1525,13 @@ class TestFernetTokenProviderV2(RestfulTestCase):
     def config_overrides(self):
         super(TestFernetTokenProviderV2, self).config_overrides()
         self.config_fixture.config(group='token', provider='fernet')
+        self.useFixture(
+            ksfixtures.KeyRepository(
+                self.config_fixture,
+                'fernet_tokens',
+                CONF.fernet_tokens.max_active_keys
+            )
+        )
 
     def test_authenticate_unscoped_token(self):
         unscoped_token = self.get_unscoped_token()
@@ -1524,6 +1598,9 @@ class TestFernetTokenProviderV2(RestfulTestCase):
         self.assignment_api.add_role_to_user_and_project(self.user_foo['id'],
                                                          project_ref['id'],
                                                          self.role_admin['id'])
+        token_id = unscoped_token
+        if six.PY2:
+            token_id = token_id.encode('ascii')
         r = self.public_request(
             method='POST',
             path='/v2.0/tokens',
@@ -1531,7 +1608,7 @@ class TestFernetTokenProviderV2(RestfulTestCase):
                 'auth': {
                     'tenantName': project_ref['name'],
                     'token': {
-                        'id': unscoped_token.encode('ascii')
+                        'id': token_id,
                     }
                 }
             },
